@@ -1,6 +1,14 @@
 # coding=utf-8
 """2D Polygon"""
 from __future__ import division
+import math
+import time
+from collections import deque
+
+try:  # Python3
+    from queue import PriorityQueue
+except ImportError:  # Python2
+    from Queue import PriorityQueue
 
 from .pointvector import Point2D, Vector2D
 from .line import LineSegment2D
@@ -9,8 +17,7 @@ from ..intersection2d import intersect_line2d, intersect_line2d_infinite, \
     does_intersection_exist_line2d, closest_point2d_on_line2d
 from ._2d import Base2DIn2D
 
-from collections import deque
-import math
+inf = float("inf")
 
 
 class Polygon2D(Base2DIn2D):
@@ -363,6 +370,75 @@ class Polygon2D(Base2DIn2D):
             is_equivalent = is_equivalent and pt.is_equivalent(other_pt, tolerance)
         return is_equivalent
 
+    def pole_of_inaccessibility(self, tolerance):
+        """Get the pole of inaccessibility for the polygon.
+
+        The pole of inaccessibility is the most distant internal point from the
+        polygon outline. It is not to be confused with the centroid, which
+        represents the "center of mass" of the polygon and may be outside of
+        the polygon if the shape is concave. The poly of inaccessibility is
+        useful for optimal placement of a text label on a polygon.
+
+        The algorithm here is a port of the polylabel library from MapBox
+        assembled by Michal Hatak. (https://github.com/Twista/python-polylabel).
+
+        Args:
+            tolerance: The precision to which the pole of inaccessibility
+                will be computed.
+        """
+        # compute the cell size from the bounding rectangle
+        min_x, min_y = self.min.x, self.min.y
+        max_x, max_y = self.max.x, self.max.y
+        width = max_x - min_x
+        height = max_y - min_y
+        cell_size = min(width, height)
+        h = cell_size / 2.0
+        if cell_size == 0:  # degenerate polygon; just return the minimum
+            return self.min
+
+        # get an array representation of the polygon and set up the priority queue
+        _polygon = tuple(pt.to_array() for pt in self.vertices)
+        cell_queue = PriorityQueue()
+
+        # cover polygon with initial cells
+        x = min_x
+        while x < max_x:
+            y = min_y
+            while y < max_y:
+                c = _Cell(x + h, y + h, h, _polygon)
+                y += cell_size
+                cell_queue.put((-c.max, time.time(), c))
+            x += cell_size
+
+        best_cell = self._get_centroid_cell(_polygon)
+
+        bbox_cell = _Cell(min_x + width / 2, min_y + height / 2, 0, _polygon)
+        if bbox_cell.d > best_cell.d:
+            best_cell = bbox_cell
+
+        # recursively iterate until we find the pole
+        num_of_probes = cell_queue.qsize()
+        while not cell_queue.empty():
+            _, __, cell = cell_queue.get()
+
+            if cell.d > best_cell.d:
+                best_cell = cell
+
+            if cell.max - best_cell.d <= tolerance:
+                continue
+
+            h = cell.h / 2
+            c = _Cell(cell.x - h, cell.y - h, h, _polygon)
+            cell_queue.put((-c.max, time.time(), c))
+            c = _Cell(cell.x + h, cell.y - h, h, _polygon)
+            cell_queue.put((-c.max, time.time(), c))
+            c = _Cell(cell.x - h, cell.y + h, h, _polygon)
+            cell_queue.put((-c.max, time.time(), c))
+            c = _Cell(cell.x + h, cell.y + h, h, _polygon)
+            cell_queue.put((-c.max, time.time(), c))
+            num_of_probes += 4
+        return Point2D(best_cell.x, best_cell.y)
+
     def remove_colinear_vertices(self, tolerance):
         """Get a version of this polygon without colinear or duplicate vertices.
 
@@ -447,6 +523,63 @@ class Polygon2D(Base2DIn2D):
         else:
             return Polygon2D(tuple(pt.scale(factor, origin) for pt in self.vertices))
 
+    def offset(self, distance, check_intersection=False):
+        """Offset the polygon by a given distance inwards or outwards.
+
+        Note that the resulting shape may be self-intersecting if the distance
+        is large enough and the is_self_intersecting property may be used to identify
+        these shapes.
+
+        Args:
+            distance: The distance inwards that the polygon will be offset.
+                Positive values will be offset inwards while negative ones
+                will be offset outwards.
+            check_intersection: A boolean to note whether the resulting operation
+                should be checked for self intersection and, if so, None will be
+                returned instead of the mis-shaped polygon.
+        """
+        # make sure the offset is not zero
+        if distance == 0:
+            return self
+
+        # loop through the vertices and get the new offset vectors
+        move_vecs, max_i = [], len(self._vertices) - 1
+        for i, pt in enumerate(self._vertices):
+            v1 = self._vertices[i - 1] - pt
+            end_i = i + 1 if i != max_i else 0
+            v2 = self._vertices[end_i] - pt
+            if not self.is_clockwise:
+                ang = v1.angle_clockwise(v2) / 2
+                m_vec = v1.rotate(-ang).normalize()
+                m_dist = distance / math.sin(ang)
+            else:
+                ang = v1.angle_counterclockwise(v2) / 2
+                m_vec = v1.rotate(ang).normalize()
+                m_dist = -distance / math.sin(ang)
+            m_vec = m_vec * m_dist
+            move_vecs.append(m_vec)
+
+        # move the vertices by the offset to create the new Polygon2D
+        new_pts = tuple(pt.move(m_vec) for pt, m_vec in zip(self._vertices, move_vecs))
+        new_poly = Polygon2D(new_pts)
+
+        # check for self intersection between the moving vectors if requested
+        if check_intersection:
+            poly_segs = new_poly.segments
+            _segs = [LineSegment2D(p, v) for p, v in zip(self._vertices, move_vecs)]
+            _skip = (0, len(_segs) - 1)
+            _other_segs = [x for j, x in enumerate(poly_segs) if j not in _skip]
+            for _oth_s in _other_segs:
+                if _segs[0].intersect_line_ray(_oth_s) is not None:  # intersection!
+                    return None
+            for i, _s in enumerate(_segs[1: len(_segs)]):
+                _skip = (i, i + 1)
+                _other_segs = [x for j, x in enumerate(poly_segs) if j not in _skip]
+                for _oth_s in _other_segs:
+                    if _s.intersect_line_ray(_oth_s) is not None:  # intersection!
+                        return None
+        return new_poly
+
     def intersect_line_ray(self, line_ray):
         """Get the intersections between this polygon and a Ray2D or LineSegment2D.
 
@@ -853,6 +986,23 @@ class Polygon2D(Base2DIn2D):
         new_polygon._is_self_intersecting = self._is_self_intersecting
         new_polygon._is_clockwise = self._is_clockwise
 
+    @staticmethod
+    def _get_centroid_cell(polygon):
+        """Get a Cell object at the centroid of the Polygon2D."""
+        area = 0
+        x = 0
+        y = 0
+        b = polygon[-1]  # prev
+        for a in polygon:
+            f = a[0] * b[1] - b[0] * a[1]
+            x += (a[0] + b[0]) * f
+            y += (a[1] + b[1]) * f
+            area += f * 3
+            b = a
+        if area == 0:
+            return _Cell(polygon[0][0], polygon[0][1], 0, polygon)
+        return _Cell(x / area, y / area, 0, polygon)
+
     @staticmethod
     def _insert_updates_in_order(polygon, polygon_updates):
         """Insert updates from the intersect_segments method into a polygon.
@@ -968,3 +1118,88 @@ class Polygon2D(Base2DIn2D):
 
     def __repr__(self):
         return 'Polygon2D ({} vertices)'.format(len(self))
+
+
+class _Cell(object):
+    """2D cell object used in certain Polygon computations (eg. pole_of_inaccessibility).
+
+    Args:
+        x: The X coordinate of the cell origin.
+        y: The Y coordinate of the cell origin.
+        h: The dimension of the cell.
+        polygon: An array representation of a Polygon2D.
+
+    Properties:
+        * x
+        * y
+        * h
+        * d
+        * max
+    """
+    __slots__ = ('x', 'y', 'h', 'd', 'max')
+
+    def __init__(self, x, y, h, polygon):
+        self.h = h
+        self.y = y
+        self.x = x
+        self.d = self._point_to_polygon_distance(x, y, polygon)
+        self.max = self.d + self.h * math.sqrt(2)
+
+    def _point_to_polygon_distance(self, x, y, polygon):
+        """Get the distance from an X,Y point to the edge of a Polygon."""
+        inside = False
+        min_dist_sq = inf
+
+        b = polygon[-1]
+        for a in polygon:
+
+            if (a[1] > y) != (b[1] > y) and \
+                    (x < (b[0] - a[0]) * (y - a[1]) / (b[1] - a[1]) + a[0]):
+                inside = not inside
+
+            min_dist_sq = min(min_dist_sq, self._get_seg_dist_sq(x, y, a, b))
+            b = a
+
+        result = math.sqrt(min_dist_sq)
+        if not inside:
+            return -result
+        return result
+
+    @staticmethod
+    def _get_seg_dist_sq(px, py, a, b):
+        """Get the squared distance from a point to a segment."""
+        x = a[0]
+        y = a[1]
+        dx = b[0] - x
+        dy = b[1] - y
+
+        if dx != 0 or dy != 0:
+            t = ((px - x) * dx + (py - y) * dy) / (dx * dx + dy * dy)
+
+            if t > 1:
+                x = b[0]
+                y = b[1]
+
+            elif t > 0:
+                x += dx * t
+                y += dy * t
+
+        dx = px - x
+        dy = py - y
+
+        return dx * dx + dy * dy
+
+    def __lt__(self, other):
+        return self.max < other.max
+
+    def __lte__(self, other):
+        return self.max <= other.max
+
+    def __gt__(self, other):
+        return self.max > other.max
+
+    def __gte__(self, other):
+        return self.max >= other.max
+
+    def __eq__(self, other):
+        return self.max == other.max
